-- (c) David Cunningham 2009, Licensed under the MIT license: http://www.opensource.org/licenses/mit-license.php

print("Loading player_ctrl.lua")

if player_ctrl == nil then
        player_ctrl = {
                fast = false,
                frameTimer = Timer(),

                mouseSensitivity = 0.06,
                mouseInvert = true,

                speedSlow = 3,
                speedFast = 100,

                forwards = 0,
                backwards = 0,
                left = 0,
                right = 0,
                up = 0,
                down = 0,

                ghostMode = false,

                shootFast = 100,
                shootSlow = 10,
                shootSpin = Vector3(0,0,30),

                currentBoomLength = gfx.camChaseBoom.position.y,
                boomRayRadius = 0.1,

                mode = 0, -- modes are 0 (ghost)  1 (drive)
                ghostBinds = BindTable.new(),
                walkBinds = BindTable.new(),
                driveBinds = BindTable.new(),
        }

else
        main.frameCallbacks:removeByName("player_ctrl")
        ui.pressCallbacks:removeByName("player_ctrl")
        ui.pointerGrabCallbacks:removeByName("player_ctrl")
        player_ctrl.speedo = safe_destroy(player_ctrl.speedo)
end

player_ctrl.speedo = get_hud_root():addChild("ShadowText",{font = "misc.fixed", charHeight = 13})
player_ctrl.speedo.resize = Hud.BOTTOM_LEFT()
player_ctrl.speedo.text = "Ghost Mode"

ui.pressCallbacks:insert("player_ctrl",function (key)
        if player_ctrl.mode == 0 then
                return player_ctrl.ghostBinds:process(key)
        elseif player_ctrl.mode == 1 then
                return player_ctrl.driveBinds:process(key)
        else
                error("player_ctrl.mode unrecognised! ("..player_ctrl.mode..")")
        end
end)

ui.pointerGrabCallbacks:insert("player_ctrl",function (rel_x,rel_y)
        local yaw, pitch

        if player_ctrl.mode == 0 then
                yaw, pitch = gfx.camYaw, gfx.camPitch
        elseif player_ctrl.mode == 1 then
                yaw, pitch = gfx.camChaseYaw, gfx.camChasePitch
        end

        local sens = player_ctrl.mouseSensitivity
        local inv = player_ctrl.mouseInvert and 1 or -1
        local rot_x, rot_y = rel_x*sens, inv*rel_y*sens

        yaw.orientation   = yaw.orientation   * Quat(rot_x,V_DOWN)
        local p = pitch.orientation * Quat(rot_y,V_RIGHT)

        if p.angle > 90 then
                local sign = p.x
                local v = Vector3(sign>0 and 1 or -1, 0, 0)
                p = Quat(90,v)
        end
        pitch.orientation = p
end)

main.frameCallbacks:insert("player_ctrl", function ()
        if player_ctrl.mode == 0 then
                player_ctrl:strafe()
        elseif player_ctrl.mode == 1 then
                player_ctrl:checkBoomLength()
                if player_ctrl.vehicle.activated == false then
                        player_ctrl:abandonVehicle()
                        return
                end
                if player_ctrl.vehicle.getSpeed ~= nil then
                        if player_ctrl.speedo ~= nil then
                                player_ctrl.speedo.text =
                                        string.format("Speed: %d MPH\n",
                                                      player_ctrl.vehicle:getSpeed()*60*60/1500)
                        end
                end
        end
end)

-- reuse object to avoid allocations
local displacement = Vector3(0,0,0)

function player_ctrl:strafe()
        local right = self.right - self.left
        local forwards = self.forwards - self.backwards
        local up = self.up - self.down

        local interval = self.frameTimer.us/1000000
        self.frameTimer:reset()
        

        local active_speed = self.fast and self.speedFast or self.speedSlow

        local dist = active_speed * interval

        -- we now know how far to move (dist)
        -- but not in which directions

        displacement:xyz(dist*right, dist*forwards, 0)
        local pain = gfx.camRoll.derivedOrientation
        displacement:rotate(pain)
        
        displacement:append(0, 0, dist*up)

        local fraction, _, normal = pw:cast(0.15, gfx.camPos.derivedPosition, gfx.camPos.derivedPosition + displacement, 0, true, true)

        if not self.ghostMode and fraction ~= nil then
                normal = normal.normalised
                displacement = fraction * displacement + ((1-fraction) * displacement - normal * (displacement:dot(normal)))
                local _, fraction2, _ =
                        pw:cast(0.15, gfx.camPos.derivedPosition, gfx.camPos.derivedPosition + displacement, 0, true, true)
                if fraction2 ~= nil then
                        return
                end
        end
        
        -- splendid, now let's move

        gfx.camPos:translate(displacement)

end

function player_ctrl:pickDrive()
        local obj = pick_obj_safe()
        if obj == nil then return end
        self:drive(obj)
end

function player_ctrl:drive(vehicle)
        if not vehicle.activated then error("Can't drive a vehicle that isn't activated") end
        self.vehicle = vehicle
        local yaw = gfx:yaw()
        gfx.cam:detach()
        gfx.cam:attach(gfx.camChaseBoom)
        if gfx.camChase.parentSceneNode then
                gfx.camChase.parentSceneNode:removeChild(gfx.camChase)
        end
        vehicle.instance.gfx:addChild(gfx.camChase)
        gfx.camChaseYaw.orientation = yaw
        gfx.camChasePitch.orientation = gfx.camPitch.orientation
        gfx.camChaseRoll.orientation = Q_ID
        self.mode = 1
end

function player_ctrl:abandonVehicle()
        if self.vehicle and self.vehicle.activated and self.vehicle.abandon ~= nil then
                self.vehicle:abandon()
        end
        self.vehicle = nil
        local pos = gfx.cam.derivedPosition
        local yaw = gfx:yaw()
        gfx.cam:detach()
        gfx.cam:attach(gfx.camRoll)
        gfx.camPos.position = pos
        gfx.camYaw.orientation = yaw
        gfx.camPitch.orientation = gfx.camChasePitch.orientation
        self.mode = 0
        self.speedo.text = "Ghost Mode"
end

function player_ctrl:checkBoomLength()
        local v = self.vehicle
        if v==nil then return end
        if not v.activated then return end
        local body = v.instance.body
        local ray_reach =
                body.world:cast(self.boomRayRadius,
                          gfx.camChaseRoll.derivedPosition,
                          gfx.camChaseBoom.derivedOrientation * V_BACKWARDS,
                          self.currentBoomLength, true, true)

        ray_reach = ray_reach or self.currentBoomLength

        gfx.camChaseBoom.position = ray_reach * V_BACKWARDS
end

function player_ctrl:zoom()
        if ui:shift() then
                self:zoomOut()
        else
                self:zoomIn()
        end
end

function player_ctrl:zoomIn()
        --self.currentBoomLength = clamp(self.currentBoomLength - 0.2,0,40)
        self.currentBoomLength = clamp(self.currentBoomLength * 0.833333333,0,40)
end

function player_ctrl:zoomOut()
        --self.currentBoomLength = clamp(self.currentBoomLength + 0.2,0,40)
        self.currentBoomLength = clamp(self.currentBoomLength * 1.200000000,0,40)
end


function player_ctrl:fire (type)
        local speed = ui:ctrl() and self.shootFast or self.shootSlow
        local spin = ui:alt() and self.shootSpin
        fire_extended(speed,type,spin)
end



function player_ctrl:flush()
        if self.mode == 0 then
                self.ghostBinds:flush()
        elseif self.mode == 1 then
                self.driveBinds:flush()
        else
                error("player_ctrl.mode unrecognised! ("..self.mode..")")
        end
end





function cam_ray()
        return pw:cast(-1,gfx.cam.derivedPosition,
                       gfx.cam.derivedOrientation * V_FORWARDS,
                       8000, true, false)
end

function test_ray()
        local old = pw.verboseCasts
        pw.verboseCasts = true
        local a,b,c,d = pw:cast(-1,gfx.cam.derivedPosition,
                       gfx.cam.derivedOrientation * V_FORWARDS,
                       8000, true, false)
        pw.verboseCasts = false
        return a,b,c,d
end

function pick_pos()
        local dist = cam_ray()
        if dist == nil then
                error("Not pointing at anything",2)
        end
        return gfx.cam.derivedPosition + gfx.cam.derivedOrientation*(dist*V_FORWARDS)
end

function pick_obj_safe()
        local _, body = cam_ray()
        if body == nil then return nil end
        return body.owner
end     

function pick_obj()
        local obj = pick_obj_safe()
        if obj == nil then
                error("Not pointing at anything",2)
        end
        return obj
end             

function fire_extended(v,t,spin)
        v = v or 40
        t = t or "Crate"
        local q = gfx.cam.derivedOrientation
        local x,y,z = gfx.cam.derivedPosition:xyz()
        local o = streamer:addObject(t,x,y,z,{rot=q, norespawn=true, projectile=true})
        o:activate() -- need this so we can add the linear velocity
        o.instance.body.linearVelocity = q * Vector3(0,v,0)
        if spin then
                o.instance.body.angularVelocity = q * spin
        end
end

function place(class,height,r)
        height = height or 0
        if r==nil then
                r = Q_ID
        else
                r = Quat(r,V_UP)
        end
        local x,y,z = pick_pos():xyz()
        streamer:addObject(class,x,y,z+height,{rot=gfx:yaw()*r, placed=true})
end

function stack(pos,quat,height,class,dim)
        pos = pos or pick_pos()
        quat = quat or gfx:yaw()
        height = height or 20
        class = class or "common/Crate/Small"
        dim = dim or 0.3
        for i=0.5,height-0.5 do
                local x,y,z = pos:xyz()
                z = z + dim*i
                streamer:addObject(class,x,y,z,{rot=quat, placed=true})
        end     
end             
                
function wall(pos,quat,x_min,x_max,height)
        pos = pos or pick_pos()
        quat = quat or gfx:yaw()
        x_min = x_min or -8
        x_max = x_max or 8
        height = height or 15
        for z=0,height-1 do
                local x_min_, x_max_ = x_min, x_max
                if z % 2 == 1 then
                        x_min_, x_max_ =x_min+1, x_max-1
                end
                for x=x_min_,x_max_,2 do
                        local px, py, pz = (pos + quat * (0.6*Vector3(x,0,z)+Vector3(0,0,0.3))):xyz()
                        streamer:addObject("common/Crate/Long",px,py,pz,{rot=quat,placed=true})
                end 
        end             
end                     

function bowling(pos,quat,rows,space) 
        pos = pos or pick_pos()
        quat = quat or gfx:yaw() 
        rows = rows or 5 
        space = space or 1.6 
        local spacex = space * 0.5 
        local spacey = space * math.sqrt(3)/2 
        for y=1,rows do 
                for x=-y+1,y,2 do 
                        local pinx = x*spacex 
                        local piny = (y-1)*spacey 
                        local pinz = 0.63 
                        local tripos = quat * Vector3(pinx,piny,pinz) + pos 
                        pinx, piny, pinz = tripos:xyz() 
                        local o = streamer:addObject("common/Bowling/Pin",pinx,piny,pinz,{placed=true}) 
                end 
        end 
end

function jenga(h,pos,quat)
        h = h or 10
        pos = pos or pick_pos()
        quat = quat or gfx:yaw()
        for level=0,h do
                for b=-1,1 do
                        local x, y, z, opts
                        if level%2==0 then
                                x, y, z, opts = 0,b*.3,(.5+level)*.24,{test=1,rot=quat*Q_FORWARDS}
                        else
                                x, y, z, opts = b*.3,0,(.5+level)*.24,{rot=quat*Q_RIGHT,test=1}
                        end
                        x,y,z = (pos+quat*Vector3(x,y,z)):xyz()
                        local o = streamer:addObject("common/Jenga/Brick",x,y,z,opts)
                end
        end
end


function clear_placed()
        for _,v in ipairs(streamer.objects) do
                if v.placed then
                        v:destroy()
                end
        end
end

function clear_projectiles()
        for _,v in ipairs(streamer.objects) do
                if v.projectile then
                        v:destroy()
                end
        end
end

function clear_norespawns()
        for _,v in ipairs(streamer.objects) do
                if v.norespawn then
                        v:destroy()
                end
        end
end

-- vim: shiftwidth=8:tabstop=8:expandtab
