local mu = 1.2
local rad = 0.20 -- wheels
--local cy=-1.37
--local cz=-0.565
local cy=-0.46
local cz=0.10

streamer:addClass("../Bonanza", extends(Plane) {
	gfxMesh="Bonanza/Body.mesh",
	colMesh="Bonanza/chassis.tcol",

	--CM Arm relative to origin. In meters. (Will be multiplied by weight to derrive torque.)
	--cmArm = Vector3(0, -2.0, -0.5),
	cmArm = Vector3(0,0,0),

	--One plot for each engine. Units: [m/s] = N (Bonanza: prop wash ~ 44m/s.)
	thrustPlots = {
		--{[0] = 5000; [30] = 2500; [100] = 0; },
		{[0] = 7000; [30] = 5000; [100] = 0; },
	},

	--Aerodynamic Surfaces. For each surface: Center of pressure, "Up" direction, glide ratio, lift curve (in N/(m/s)^2). Sin(AoA) used instead of AoA.
	surfaceInfo = {
		--Left Wing.
		[1] = {
			cpr = Vector3( -4, -0.3, 0.3);
			act = Vector3(0.2,  0.0, .98);
			glide = 7;
			--lift = {[-.3] = 0; [-.2] = -3.0; [0.3] = 6.5; [0.4] = 0;};
			lift = {[-2]=0; [-1]=0; [-.3] = 0; [-.2] = -7; [0.3] = 20; [0.4] = 0; [1]=0; [2]=0;};
			drag = {[-1] = 3.0; [0.0] = 7.0; [0.3] = 20; [1.0] = 200; [2]=200;};
			limitAoA = {-.3; .4;};
			ctype = "lwing";
		};
		--Right Wing.
		[2] = {
			cpr = Vector3(4, -0.3, 0.3);
			act = Vector3(-0.2,  0.0, .98);
			glide = 7;
			--lift = {[-.3] = 0; [-.2] = -3.0; [0.3] = 6.5; [0.4] = 0;};
			lift = {[-2]=0; [-1]=0; [-.3] = 0; [-.2] = -7; [0.3] = 20; [0.4] = 0; [1]=0; [2]=0;};
			drag = {[-1] = 3.0; [0.0] = 7.0; [0.3] = 20; [1.0] = 200; [2]=200;};
			limitAoA = {-.3; .4;};
			ctype = "rwing";
		};
		--Tail Wing.
		[3] = {
			cpr = Vector3( 0,  -6.0, 0.5);
			act = Vector3( 0,  0.0, 1.0);
			glide = 7;
			--lift = {[-.4] = 0; [-.3] = -1.0; [0.2] = 0.4; [0.3] = 0;};
			lift = {[-2]=0; [-1]=0; [-.4] = 0; [-.3] = -7.0; [0.2] = 3; [0.3] = 0; [1]=0; [2]=0;};
			drag = {[-1] = 0.1; [0.0] = 0.2; [0.3] = 3.0; [1.0] = 20; [2] = 20;};
			limitAoA = {-.4; .3;};
			ctype = "elevator";
		};
		--Tail Stabilizer.
		[4] = {
			cpr = Vector3( 0.0, -6.0, 0.5);
			act = Vector3( -1.0, 0.0, 0.0);
			glide =7;
			lift = {[-2]=0; [-1]=0; [-.35]=0; [-.25] = - 3.0; [0.25] = 3.0; [0.35] = 0; [1]=0; [2]=0;};
			drag = {[-1] = 1.0; [0.0] = 1.0; [0.25] = 3.0; [1.0] = 15.0; [2] = 15;};
			limitAoA = {-.35; .35;};
			ctype = "rudder";
		}
	},

	--Lights
	lightInfo = {
		[1] = {
			pos = Vector3(-4.7,-0.2,0.4);
			rad = 1.0;
			color = Vector3(1,0,0);
		};
		[2] = {
			pos = Vector3( 4.7,-0.2,0.4);
			rad = 1.0;
			color = Vector3(0,1,0);
		};
		[3] = {
			pos = Vector3( 0,-5.35,2);
			rad = 1.0;
			color = Vector3(1,1,1);
		};
	};

	--Rotors.
	rotorInfo = { },

	--Drag
	dragInfo = { },

	--Exactly the same structure as in vehicle class.
	wheelInfo = {
                {boned=true, pos=Vector3(0.0, 1.444, -0.406),
                 left=true; steer=1; drive=false; handbrake=false;
                 len=0.15; rad=rad; slack=0.16;
                 mu = mu; strutBone="gear_strut_f", axleBone="gear_axle_f", wheelBone="gear_wheel_f",
                 down = V_DOWN*Quat(15,V_RIGHT);
                },

                {boned=true, pos=Vector3(1.745, -0.726, -0.1291);
                 left=true; steer=0; drive=false; handbrake=true;
                 len=0.11; rad=rad; slack=0.22;
                 mu = mu; strutBone="gear_strut_s", axleBone="gear_axle_s", wheelBone="gear_wheel_s",
                },

                {boned=true, pos=Vector3(-1.745, -0.726, -0.1291);
                 left=false; steer=0; drive=false; handbrake=true;
                 len=0.11; rad=rad; slack=0.22;
                 mu = mu; strutBone="gear_strut_p", axleBone="gear_axle_p", wheelBone="gear_wheel_p",
                },
        },

    gearDoors = { gear_door_p   = Plot{[0]=0; [1]=100},
                  gear_door_s   = Plot{[0]=0; [1]=100},
                  gear_door_f_p = Plot{[0]=0; [0.35]=120; [0.350001]=120, [1]=120},
                  gear_door_f_s = Plot{[0]=0; [0.35]=120; [0.350001]=120, [1]=120},
    },

    gearStruts = { gear_strut_p = Plot{[0]=0; [0.2]=0; [0.200001]=0; [1]=80},
                   gear_strut_s = Plot{[0]=0; [0.2]=0; [0.200001]=0; [1]=80},
                   gear_strut_f = Plot{[0]=0; [0.2]=0; [0.250001]=0; [1]=110},
    },

    gearCycle = 4,

},{})

streamer:addClass("../BonanzaChassis", ColClass, { gfxMesh="Bonanza/chassis.mesh", colMesh="Bonanza/chassis.tcol" })

material "grey" 	{ diffuseMap = "Defaul.jpg"; specularColour = {0.898039, 0.898039, 0.898039}; specularity = 55; }
material "white" 	{ diffuseColour = {0.360784, 0.360784, 0.360784}; specularColour = {0.898039, 0.898039, 0.898039}; specularity = 1; }
material "anthracite" 	{ diffuseColour = {0.0980392, 0.0980392, 0.0980392}; specularColour = {0.197569, 0.197569, 0.197569}; specularity = 25; }
material "red" 		{ diffuseMap = "red.jpg"; specularColour = {0.898039, 0.898039, 0.898039}; specularity = 55; }
material "grey" 	{ diffuseColour = {0.552941, 0.552941, 0.552941}; specularColour = {0.044902, 0.044902, 0.044902}; specularity = 25; }
material "tyre" 	{ diffuseColour = {0.1, 0.1, 0.1}; }
material "trans" 	{ alpha=0.3; backfaces=true; diffuseColour = {0.0980392, 0.0980392, 0.0980392}; specularColour = {0.044902, 0.044902, 0.044902}; specularity = 66; }
material "fin" 		{ diffuseMap = "fin.jpg"; specularColour = {0.898039, 0.898039, 0.898039}; specularity = 55; }
material "nose" 	{ diffuseMap = "nose.jpg"; specularColour = {0.898039, 0.898039, 0.898039}; specularity = 55; }
material "fus" 		{ diffuseMap = "fus.jpg"; specularColour = {0.898039, 0.898039, 0.898039}; specularity = 55; }
material "panel" 	{ diffuseMap = "panel.jpg"; specularColour = {0.044902, 0.044902, 0.044902}; specularity = 25; }
material "tail" 	{ diffuseMap = "tail.jpg"; specularColour = {0.898039, 0.898039, 0.898039}; specularity = 55; }
material "lwin" 	{ diffuseMap = "lwin.jpg"; specularColour = {0.898039, 0.898039, 0.898039}; specularity = 55; }
material "rwin" 	{ diffuseMap = "rwin.jpg"; specularColour = {0.898039, 0.898039, 0.898039}; specularity = 55; }

function gear_doors_open (o, amount)
    local e = o.instance.entity
    for _,b in ipairs{e:getBoneId("gear_door_f_p"), e:getBoneId("gear_door_f_s")} do
        e:setBoneOrientation(b,(Quat(e:getBoneInitialOrientation(b))*Quat(lerp(amount,0,130),V_FORWARDS)):wxyz())
    end
end

function gear_down (o, amount)
    local e = o.instance.entity
    local bone = e:getBoneId("gear_strut_f")
    e:setBoneOrientation(bone,(Quat(e:getBoneInitialOrientation(bone))*Quat(lerp(amount,0,-110),Vector3(1,0,0))):wxyz())
end

